///////////////////////////////////////////////////////////////////////////////
// File    : interaction.h
// Content : interaction technique encapsulation
// History : 06-Mar-2006 - [rb] creation
//           19-Oct-2007 - [rb] added CeCIIL licence, prepared distro
///////////////////////////////////////////////////////////////////////////////

/* licence ********************************************************************
Copyright Renaud Blanch [http://iihm.imag.fr/blanch/]

This software is a computer program whose purpose is to explore new interaction
techniques for zoomable treemaps.

This software is governed by the CeCILL license under French law and abiding by 
the rules of distribution of free software. You can  use, modify and/ or 
redistribute the software under the terms of the CeCILL license as circulated  
at the following URL [http://www.cecill.info]. 

As a counterpart to the access to the source code and  rights to copy, modify 
and redistribute granted by the license, users are provided only with a limited 
warranty  and the software's author,  the holder of the economic rights,  and 
the successive licensors  have only limited liability. 

The fact that you are presently reading this means that you have had knowledge 
of the CeCILL license and that you accept its terms.
******************************************************************************/

#if !defined(INTERACTION_H)
#define INTERACTION_H


/* includes ******************************************************************/

#include <vector>
#include <string>


/* forward declaration *******************************************************/

class Treemap;
class View;


/* Interaction declaration ***************************************************/

class Interaction {
	static Interaction *interaction;
	typedef std::vector< double > Gesture; 
	static Gesture gesture;
	
protected:
	const Treemap &treemap;
	View &view;
	
	const std::string command;
	
protected :	
	enum ButtonState {
		UP, DOWN, DRAG
	};
	
	// buttons state
	ButtonState left;	
	ButtonState middle;
	ButtonState right;
	mutable int wheel;

	// last recorded position
	int Xm, Ym; // mouse
	int Xl, Yl; // left button down
	int Xr, Yr; // right button down
	
	const Treemap *current;
	const Treemap *selection;

public :
	Interaction(const Treemap &t, View &v, const std::string &c);
	virtual ~Interaction();
	
	virtual void renderFeedback() const;
	
	static void mouse(int button, int state, int X, int Y) {
		interaction->handleButton(button, state, X, Y); }
	static void motion(int X, int Y) {
		interaction->handleMotion(X, Y); }

	inline void up(int X, int Y)    { zoom(X, Y,  5); }
	inline void down(int X, int Y)  { zoom(X, Y, -5); }

	inline void select(int X, int Y) { select_in(X, Y); }
	inline void back(int X, int Y)   { select_out(X, Y); }
	
protected :
	// input
	void handleButton(int button, int state, int X, int Y);
	void handleMotion(int X, int Y);
	
	// select
	virtual void select_in(int X, int Y);
	virtual void select_out(int X, int Y);

	// pan & zoom
	virtual void pan(int dX, int dY);
	virtual void zoom(int X, int Y, double dz = 0.);

	// stroke
	virtual void stroke_begin(int X, int Y);
	virtual void stroke(int X, int Y);
	virtual void stroke_end(int X, int Y);	
};


#endif // !defined(INTERACTION_H)
