///////////////////////////////////////////////////////////////////////////////
// File    : treemap.h
// Content : basic treemap structure
// History : 10-Apr-2006 - [rb] creation
//           19-Oct-2007 - [rb] added CeCIIL licence, prepared distro
///////////////////////////////////////////////////////////////////////////////

/* licence ********************************************************************
Copyright Renaud Blanch [http://iihm.imag.fr/blanch/]

This software is a computer program whose purpose is to explore new interaction
techniques for zoomable treemaps.

This software is governed by the CeCILL license under French law and abiding by 
the rules of distribution of free software. You can  use, modify and/ or 
redistribute the software under the terms of the CeCILL license as circulated  
at the following URL [http://www.cecill.info]. 

As a counterpart to the access to the source code and  rights to copy, modify 
and redistribute granted by the license, users are provided only with a limited 
warranty  and the software's author,  the holder of the economic rights,  and 
the successive licensors  have only limited liability. 

The fact that you are presently reading this means that you have had knowledge 
of the CeCILL license and that you accept its terms.
******************************************************************************/

#if !defined(TREEMAP_H)
#define TREEMAP_H


/* includes ******************************************************************/

#include <string>
#include <map>
#include <istream>

#include "rectangle.h"


/* forward declarations ******************************************************/

class Placer;
class Renderer;


/* tree declaration **********************************************************/

class Treemap : Rectangle {
protected:
	// structure	
	const Treemap *parent; 
	typedef std::map< std::string, Treemap * > ChildMap;
	ChildMap children;

	// attributes
	double weight;
	int level;
	
public:
	const std::string label;
	const Rectangle &box;
	
private:
	// disabling copy
	Treemap(const Treemap &);
	const Treemap& operator=(const Treemap &);

	// tree construction
	Treemap(const Treemap *p = 0, const std::string &l = "", double w = 0.) :
		Rectangle(), parent(p), weight(w),
		level(parent->level + 1),
		label(l), box(*this) {
	}	 


protected:
	void append(std::string path, double weight);

	void do_place(const Placer *placer);
	void do_render(const Renderer *renderer) const;

	void do_print(std::ostream &os) const;


public:
	Treemap(std::istream &input);
	virtual ~Treemap();

	// navigation
	const Treemap *up() const { return parent; }
	const Treemap *down(double x, double y) const;
	const Treemap *pick(double x, double y) const;
	
	std::string path() const;

	// layout & display
	void place(const Placer *placer);
	void render(const Renderer *renderer) const;

	// output
	void print(std::ostream &os) const;
	friend std::ostream &operator<<(std::ostream &os, const Treemap &t) {
		t.print(os);
		return os;
	}	
};


#endif // !defined(TREEMAP_H)

