///////////////////////////////////////////////////////////////////////////////
// File    : utils.h
// Content : os dependent utility functions
// History : 05-Jul-2006 - [rb] creation
//           19-Oct-2007 - [rb] added CeCIIL licence, prepared distro
///////////////////////////////////////////////////////////////////////////////

/* licence ********************************************************************
Copyright Renaud Blanch [http://iihm.imag.fr/blanch/]

This software is a computer program whose purpose is to explore new interaction
techniques for zoomable treemaps.

This software is governed by the CeCILL license under French law and abiding by 
the rules of distribution of free software. You can  use, modify and/ or 
redistribute the software under the terms of the CeCILL license as circulated  
at the following URL [http://www.cecill.info]. 

As a counterpart to the access to the source code and  rights to copy, modify 
and redistribute granted by the license, users are provided only with a limited 
warranty  and the software's author,  the holder of the economic rights,  and 
the successive licensors  have only limited liability. 

The fact that you are presently reading this means that you have had knowledge 
of the CeCILL license and that you accept its terms.
******************************************************************************/

/* includes ******************************************************************/

#include "utils.h"

#include <cassert>
using namespace std;


/* open via shell script *****************************************************/

#define UNIX_LIKE
#if defined(UNIX_LIKE)

#include <cstdlib>  // exit
#include <unistd.h> // fork & execl

void open(const char *command, const char *path) {
	if(fork() == 0) {
		int status = execl(command, command, path, 0);
		// reached only upon failure of the previous call
		assert(status == -1);
		exit(status);
	}
}

#else

void open(const char *command, const char *path) {
	cout << path << endl;
}

#endif // defined(UNIX_LIKE)


/* make the application front most *******************************************/

#if defined(__APPLE__)

#include <Carbon/Carbon.h>

void bring_front() {
	static const ProcessSerialNumber psn = { 0, kCurrentProcess };
	SetFrontProcess(&psn);
}

#else

void bring_front() {}

#endif // defined(__APPLE__)
